/*--------------------------------------------------------------------------
 *  Copyright 2008 utgenome.org
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *--------------------------------------------------------------------------*/
//--------------------------------------
// utgb-shell Project
//
// BEDDatabaseGenerator.java
// Since: May 26, 2009
//
// $URL: http://svn.utgenome.org/utgb/tags/utgb/utgb-shell/utgb-shell-1.2.9/src/main/java/org/utgenome/shell/db/bed/BEDDatabaseGenerator.java $ 
// $Author: leo $
//--------------------------------------
package org.utgenome.shell.db.bed;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.io.Reader;
import java.io.Serializable;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Statement;

import org.utgenome.shell.db.Gene;
import org.xerial.core.XerialException;
import org.xerial.lens.Lens;
import org.xerial.util.log.Logger;

public class BEDDatabaseGenerator {

	private static Logger _logger = Logger.getLogger(BEDDatabaseGenerator.class);

	public static void toSQLiteDB(Reader bedInput, String dbName) throws IOException, XerialException {
		BED2SilkReader reader = new BED2SilkReader(bedInput);

		DBBuilder query;
		try {

			Class.forName("org.sqlite.JDBC");
			Connection conn = DriverManager.getConnection("jdbc:sqlite:" + dbName);

			query = new DBBuilder(conn);
			Lens.loadSilk(query, reader);

			query.dispose();
		}
		catch (Exception e) {
			_logger.error(e);
		}
	}

	public static class DBBuilder {

		Connection conn;
		PreparedStatement p1;
		PreparedStatement p2;
		Statement stat;

		public DBBuilder(Connection conn) throws SQLException {
			this.conn = conn;
			this.stat = conn.createStatement();

			stat.executeUpdate("pragma synchronous = off");

			conn.setAutoCommit(false);

			stat.executeUpdate("drop table if exists track");
			stat.executeUpdate("drop table if exists gene");
			stat.executeUpdate("create table track (object blob)");
			stat.executeUpdate("create table gene (coordinate text, start integer, end integer)");
			stat.executeUpdate("create index gene_index on gene (coordinate, start)");

			p1 = conn.prepareStatement("insert into track values(?)");
			p2 = conn.prepareStatement("insert into gene values(?, ?, ?)");

		}

		public void dispose() throws SQLException {

			conn.commit();

			p1.close();
			p2.close();
			stat.close();
			conn.close();
		}

		public void addTrack(BEDTrack track) {
			// store track data to db
			try {
				ByteArrayOutputStream buf = new ByteArrayOutputStream();
				ObjectOutputStream out = new ObjectOutputStream(buf);
				out.writeObject(track);
				out.flush();

				p1.setBytes(1, buf.toByteArray());
				p1.execute();
			}
			catch (Exception e) {
				_logger.error(e);
			}
		}

		public void addGene(BEDGene gene) {
			try {
				// ByteArrayOutputStream buf = new ByteArrayOutputStream();
				// ObjectOutputStream out = new ObjectOutputStream(buf);
				// out.writeObject(gene);
				// out.flush();

				p2.setString(1, gene.coordinate);
				p2.setLong(2, gene.start);
				p2.setLong(3, gene.end);
				// p2.setBytes(4, buf.toByteArray());

				p2.execute();
			}
			catch (Exception e) {
				_logger.error(e);
			}
		}
	}

	public static class BEDTrack implements Serializable {
		public String name;
		public String description;
		public int visibility;
		public String color;
		public String itemRgb;
		public int useScore;
		public String group;
		public String priority;
		public String db;
		public long offset;
		public String url;
		public String htmlUrl;

		@Override
		public String toString() {
			return String
					.format(
							"track:name=%s, description=%s, visibility=%d, color=%s, itemRgb=%s, useScore=%d, group=%s, priority=%s, db=%s, offset=%d, url=%s, htmlUrl=%s\n",
							name, description, visibility, color, itemRgb, useScore, group, priority, db, offset, url, htmlUrl);
		}
	}

	public static class BEDGene extends Gene {
		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;
		public String coordinate;

		@Override
		public String toString() {
			return String.format("%s: %s:%d-%d\t%s\t%s\t%s\t%s\n", getName(), coordinate, getStart(), getEnd(), getStrand(), getCDS(), getExon(), getColor());
		}
	}

}
